#chtNounConstruct.py
#A plugin allowing toggling on and off the Chinese character definition and noun construction functionality.
#Written by Michael Chen <m11chen@gmail.com>

import globalPluginHandler
import ui
import config
from config import conf
import speech


confSect = conf.get("chtNounConstruct")

if not confSect:
	confSect = conf["chtNounConstruct"] = {}
conf["chtNounConstruct"]["Enabled"] = "Off"

def _customSpeakSpellingGen(text,locale,useCharacterDescriptions):

	if conf["chtNounConstruct"]["Enabled"] == "On":
		useCharacterDescriptions = True

	textLength=len(text)
	synth=speech.getSynth()
	synthConfig=config.conf["speech"][synth.name]
	for count,char in enumerate(text): 
		uppercase=char.isupper()
		charDesc=None
		if useCharacterDescriptions:
			from characterProcessing import getCharacterDescription
			charDesc=getCharacterDescription(locale,char.lower())
		if charDesc:
			char=charDesc
		else:
			char=speech.processSymbol(char)
		if uppercase and synthConfig["sayCapForCapitals"]:
			char=_("cap %s")%char
		if uppercase and synth.isSupported("pitch") and synthConfig["raisePitchForCapitals"]:
			oldPitch=synthConfig["pitch"]
			synth.pitch=max(0,min(oldPitch+synthConfig["capPitchChange"],100))
		index=count+1
		speech.log.io("Speaking character %r"%char)
		if len(char) == 1 and synthConfig["useSpellingFunctionality"]:
			synth.speakCharacter(char,index=index)
		else:
			synth.speakText(char,index=index)
		if uppercase and synth.isSupported("pitch") and synthConfig["raisePitchForCapitals"]:
			synth.pitch=oldPitch
		while textLength>1 and (speech.isPaused or speech.getLastSpeechIndex()!=index):
			yield
			yield
		if uppercase and  synthConfig["beepForCapitals"]:
			speech.tones.beep(2000,50)


class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	def __init__(self):
		super(GlobalPlugin, self).__init__()
		self._oldSpeakSpellingGen = speech._speakSpellingGen
		speech._speakSpellingGen = _customSpeakSpellingGen

	def script_ToggleNounConstruction(self,gesture):
		if conf["chtNounConstruct"]["Enabled"] == "Off":
			onOff=_("On")
			conf["chtNounConstruct"]["Enabled"] = "On"
		else:
			onOff=_("Off")
			conf["chtNounConstruct"]["Enabled"] = "Off"
		ui.message(_("noun construction")+" "+onOff)
	script_ToggleNounConstruction.__doc__=_("Toggles on and off the over-riding of the _speakSpellingGen function to allow for character descriptions to be spoken when moving the system caret and review cursor.")

	def terminate(self):
		speech._speakSpellingGen = self._oldSpeakSpellingGen
		super(GlobalPlugin, self).terminate()

#: Dictionary for key bindings
	__gestures = {
		"kb:NVDA+0": "ToggleNounConstruction",
	}
